package sparkSQL;

import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;
import scala.Tuple3;
import utils.OutletParser;

/**
 * The d14_filtered.csv dataset contains recordings made at intervals of 20 seconds by sensors placed inside houses.
 * Each line has the format: id, timestamp, value, property, plug_id, household_id, house_id
 *
 * The schema of the base stream is following:
 * - id: a unique identifier of the measurement [32-bit unsigned integer value]
 * - timestamp: timestamp of measurement (number of seconds since January 1, 1970, 00:00:00 GMT) [32-bit unsigned int]
 * - value: the measurement [32-bit float]
 * - property: type of the measurement: 0 for work or 1 for load [boolean]
 * - plug_id: a unique identifier (within a household) of the smart plug [32-bit unsigned int]
 * - household_id: a unique identifier of a household (within a house) where the plug is located [32-bit unsigned int]
 * - house_id: a unique identifier of a house where the household with the plug is located [32-bit unsigned int]
 *
 * Query1: locate houses with instant power consumption greater than or equal to 350 watts.
 */
public class Debs14Query1Runner {
    private static final String pathToFile = "data/d14_filtered.csv";

    public static void main (String[] args) {

        SparkConf conf = new SparkConf()
                .setMaster("local")
                .setAppName("Query 1");
        JavaSparkContext sc = new JavaSparkContext(conf);
        sc.setLogLevel("ERROR");

        /* Preprocess data: we read the file, filter out unnecessary information, and prepare data for our processing */
        JavaRDD<String> energyFile = sc.textFile(pathToFile);
        JavaRDD<Tuple3<String, String, Double>> value = energyFile
                /* Parse data  */
                .map(line -> OutletParser.parseCSV(line))
                /* Retain only load measurements */
                .filter(x -> x != null && x.getProperty().equals("1"))
                /* Project only a subset of data */
                .map(x -> new Tuple3<>(x.getHouse_id(), x.getTimestamp(), Double.parseDouble(x.getValue())));

        System.out.println("Query 1 Spark SQL");

        /* Query 1: we import data for processing with SparkSQL and perform the query using a SQL-like syntax */
        Query1SparkSQL.process(value);

        sc.close();
    }
}
